#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_base import Node_Base
from homie.node.property.property_speed import Property_Speed
import logging

logger = logging.getLogger(__name__)


class Device_Speed(Device_Base):
    def __init__(
        self,
        device_id=None,
        name=None,
        homie_settings=None,
        mqtt_settings=None,
        speeds="OFF,LOW,MEDIUM,HIGH",
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        node = Node_Base(self, id="speed", name="Speed", type_="speed")
        self.add_node(node)

        self.speed_property = Property_Speed(
            node, data_format=speeds, set_value=self.set_speed
        )
        node.add_property(self.speed_property)

        self.start()

    def update_speed(self, speed):
        self.speed_property.value = speed
        logger.debug("Speed Update {}".format(speed))

    def set_speed(self, speed):
        logger.debug("Speed Set {}".format(speed))




#!/usr/bin/env python

import logging

from homie.device_status import Device_Status
from homie.node.property.property_temperature import Property_Temperature

logger = logging.getLogger(__name__)


class Device_Temperature(Device_Status):
    def __init__(
        self,
        device_id=None,
        name=None,
        homie_settings=None,
        mqtt_settings=None,
        temp_units="F",
    ):
        self.temp_units = temp_units

        super().__init__(device_id, name, homie_settings, mqtt_settings)

    def register_status_properties(self, node):
        self.temperature = Property_Temperature(node, unit=self.temp_units)
        node.add_property(self.temperature)

    def update_temperature(self, temperature):
        logger.info("Updated Temperature {}".format(temperature))
        self.temperature.value = temperature




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_boolean import Node_Boolean
import logging

logger = logging.getLogger(__name__)


class Device_Boolean(Device_Base):
    def __init__(
        self, device_id=None, name=None, homie_settings=None, mqtt_settings=None
    ):
        super().__init__(device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Boolean(self, id="boolean", set_boolean=self.set_boolean))

        self.start()

    def update_boolean(self, boolean):  # sends updates to clients
        self.get_node("boolean").update_boolean(boolean)
        logger.debug("Boolean Update {}".format(boolean))

    def set_boolean(self, boolean):  # received commands from clients
        logger.debug("Boolean Set {}".format(boolean))




#!/usr/bin/env python

import paho.mqtt.client as mqtt_client
import asyncio
import traceback
import threading
import functools

from uuid import getnode as get_mac
from homie.mqtt.mqtt_base import MQTT_Base

import logging

logger = logging.getLogger(__name__)

mqtt_logger = logging.getLogger("MQTT")
mqtt_logger.setLevel("INFO")


COONNECTION_RESULT_CODES = {
    0: "Connection successful",
    1: "Connection refused - incorrect protocol version",
    2: "Connection refused - invalid client identifier",
    3: "Connection refused - server unavailable",
    4: "Connection refused - bad username or password",
    5: "Connection refused - not authorised",
}


class PAHO_MQTT_Client(MQTT_Base):
    def __init__(self, mqtt_settings):
        MQTT_Base.__init__(self, mqtt_settings)

        self.mqtt_client = None

    def connect(self):
        MQTT_Base.connect(self)

        self.mqtt_client = mqtt_client.Client(
            client_id=self.mqtt_settings["MQTT_CLIENT_ID"]
        )
        self.mqtt_client.on_connect = self._on_connect
        self.mqtt_client.on_message = self._on_message
        # self.mqtt_client.on_publish = self._on_publish
        self.mqtt_client.on_disconnect = self._on_disconnect
        #self.mqtt_client.enable_logger(mqtt_logger)
        #self.mqtt_client.enable_logger()

        if self.mqtt_settings["MQTT_USERNAME"]:
            self.mqtt_client.username_pw_set(
                self.mqtt_settings["MQTT_USERNAME"],
                password=self.mqtt_settings["MQTT_PASSWORD"],
            )

        try:
            self.mqtt_client.connect(
                self.mqtt_settings["MQTT_BROKER"],
                port=self.mqtt_settings["MQTT_PORT"],
                keepalive=self.mqtt_settings["MQTT_KEEPALIVE"],
            )

            self.mqtt_client.loop_start()

        except Exception as e:
            logger.warning("MQTT Unable to connect to Broker {}".format(e))


        MQTT_Base.connect(self)

        self.mqtt_client.on_connect = self._on_connect
        self.mqtt_client.on_message = self._on_message
        self.mqtt_client.on_disconnect = self._on_disconnect


        if self.mqtt_settings["MQTT_USERNAME"]:
            self.mqtt_client.set_auth_credentials(
                self.mqtt_settings["MQTT_USERNAME"],
                self.mqtt_settings["MQTT_PASSWORD"],
            )

        def start():
            try:
                logger.warning ('Publisher loop')
                asyncio.set_event_loop(self.event_loop)
                logger.warning ('Looping forever')
                self.event_loop.run_forever()
                logger.warning ('Event loop stopped')

            except Exception as e:
                logger.error ('Error in event loop {}'.format(e))

        self.event_loop = asyncio.new_event_loop()

        logger.warning("Starting MQTT publish thread")
        self._ws_thread = threading.Thread(target=start, args=())

        self._ws_thread.daemon = True
        self._ws_thread.start()

    def publish(self, topic, payload, retain, qos):
        MQTT_Base.publish(self, topic, payload, retain, qos)

        wrapped = functools.partial(
            self.mqtt_client.publish,topic, payload, retain=retain, qos=qos
        )
        self.event_loop.call_soon(wrapped)

    def subscribe(self, topic, qos):  # subclass to provide
        MQTT_Base.subscribe(self, topic, qos)
        self.mqtt_client.subscribe(topic, qos)

    def unsubscribe(self, topic):  # subclass to provide
        MQTT_Base.unsubscribe(self, topic)
        self.mqtt_client.unsubscribe(topic)

    def set_will(self, will, topic, retain, qos):
        MQTT_Base.set_will(self, will, topic, retain, qos)
        self.mqtt_client.will_set(will, topic, retain, qos)

    def _on_connect(self, client, userdata, flags, rc):
        logger.debug("MQTT On Connect: {}".format(rc))
        self.mqtt_connected = rc == 0

    def _on_message(self, client, userdata, msg):
        topic = msg.topic
        payload = msg.payload.decode("utf-8")
        MQTT_Base._on_message(self, topic, payload, msg.retain, msg.qos)

    def _on_disconnect(self, client, userdata, rc):
        self.mqtt_connected = False  # note, change this uses the property setter, do not really need to catch this in the base class

        if rc > 0:  # unexpected disconnect
            rc_text = "Unknown result code {}".format(rc)
            if rc in COONNECTION_RESULT_CODES:
                rc_text = COONNECTION_RESULT_CODES[rc]

            logger.warning(
                "MQTT Unexpected disconnection  {} {} {}".format(
                    client, userdata, rc_text
                )
            )
        MQTT_Base._on_disconnect(self, rc)




from .property_base import Property_Base
import logging

logger = logging.getLogger(__name__)


class Property_Integer(Property_Base):
    def __init__(
        self,
        node,
        id,
        name,
        settable=True,
        retained=True,
        qos=1,
        unit=None,
        data_type="integer",
        data_format=None,
        value=None,
        set_value=None,
        tags=[],
        meta={},
    ):
        super().__init__(
            node,
            id,
            name,
            settable,
            retained,
            qos,
            unit,
            "integer",
            data_format,
            value,
            set_value,
            tags,
            meta,
        )

        if data_format:
            range = data_format.split(":")
            self.low_value = int(range[0])
            self.high_value = int(range[1])
        else:
            self.low_value = None
            self.high_value = None

    def validate_value(self, value):
        valid = True

        if self.low_value is not None and value < self.low_value:
            valid = False
        if self.high_value is not None and value > self.high_value:
            valid = False

        return valid

    def get_value_from_payload(self, payload):
        try:
            return int(payload)
        except:
            return None




from homie.device_base import Device_Base
from homie.node.node_base import Node_Base


class Device_Status(Device_Base):
    def __init__(
        self, device_id=None, name=None, homie_settings=None, mqtt_settings=None
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        node = Node_Base(self, "status", "Status", "status")
        self.add_node(node)

        self.register_status_properties(node)

        self.start()

    def register_status_properties(self, node):
        raise RuntimeError("Override in subclass")




from .property_boolean import Property_Boolean


tags = ["Contact"]


class Property_Contact(Property_Boolean):
    def __init__(
        self,
        node,
        id="contact",
        name="Contact",
        settable=False,
        retained=True,
        qos=1,
        unit=None,
        data_type=None,
        data_format=None,
        value=None,
        set_value=None,
        tags=tags,
        meta={},
    ):
        super().__init__(
            node,
            id,
            name,
            settable,
            retained,
            qos,
            unit,
            data_type,
            data_format,
            value,
            set_value,
            tags,
            meta,
        )

    def validate_value(self, value):
        return value in ["OPEN", "CLOSED"]

    def get_payload_from_value(self, value):
        if value == "OPEN":
            return "true"
        else:
            return "false"


"""
from .property_enum import Property_Enum

class Property_Contact(Property_Enum):

    def __init__(self, node, id='contact', name='Contact', settable=False, retained=False, qos=1, unit=None, data_type='enum', data_format='OPEN,CLOSED', value=None, set_value=None, tags=[], meta={}):
        
        super().__init__(node, id,name,settable,retained,qos,unit,data_type,data_format,value,set_value)

 """



import asyncio
import os
import signal
import time

from gmqtt import Client as MQTTClient


STOP = asyncio.Event()


def on_connect(client, flags, rc, properties):
    print('Connected')
    client.subscribe('homie/#', qos=0)


def on_message(client, topic, payload, qos, properties):
    print('RECV MSG:', topic, payload, qos, properties)


def on_disconnect(client, packet, exc=None):
    print('Disconnected')

def on_subscribe(client, mid, qos, properties):
    print('SUBSCRIBED')

def ask_exit(*args):
    STOP.set()

async def main(broker_host):
    client = MQTTClient("client-id")

    client.on_connect = on_connect
    client.on_message = on_message
    client.on_disconnect = on_disconnect
    client.on_subscribe = on_subscribe

    #client.set_auth_credentials(token, None)
    await client.connect(broker_host)

    client.publish('TEST/TIME', str(time.time()), qos=1)

    await STOP.wait()
    await client.disconnect()


if __name__ == '__main__':
    loop = asyncio.get_event_loop()

    host = 'openhab'
    #token = os.environ.get('FLESPI_TOKEN')

#    loop.add_signal_handler(signal.SIGINT, ask_exit)
#    loop.add_signal_handler(signal.SIGTERM, ask_exit)

    loop.run_until_complete(main(host))


#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_dimmer import Node_Dimmer
import logging

logger = logging.getLogger(__name__)
logger.setLevel("INFO")


class Device_Dimmer(Device_Base):
    def __init__(
        self, device_id=None, name=None, homie_settings=None, mqtt_settings=None
    ):
        super().__init__(device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Dimmer(self, id="dimmer", set_dimmer=self.set_dimmer))

        self.start()

    def update_dimmer(self, percent):  # sends updates to clients
        self.get_node("dimmer").update_dimmer(round(percent, 0))
        logger.debug("Dimmer Update {}".format(percent))

    def set_dimmer(self, percent):  # received commands from clients
        logger.debug("Dimmer Set {}".format(round(percent, 0)))



from .node_base import Node_Base

from homie.node.property.property_dimmer import Property_Dimmer


class Node_Dimmer(Node_Base):
    def __init__(
        self,
        device,
        id="dimmer",
        name="Dimmer",
        type_="dimmer",
        retain=True,
        qos=1,
        set_dimmer=None,
    ):
        super().__init__(device, id, name, type_, retain, qos)

        assert set_dimmer  # must provide a function to set the value of the dimmer

        self.add_property(Property_Dimmer(self, set_value=set_dimmer))

    def update_dimmer(self, percent):
        self.get_property("dimmer").value = percent




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_state import Node_State

import logging

logger = logging.getLogger(__name__)


class Device_State(Device_Base):
    def __init__(
        self,
        device_id=None,
        name=None,
        homie_settings=None,
        mqtt_settings=None,
        state_values=None,
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        self.add_node(
            Node_State(
                self,
                id="state",
                name="State",
                state_values=state_values,
                set_state=self.set_state,
            )
        )

        self.start()

    def update_state(self, state):
        self.get_node("state").update_state(state)
        logger.debug("State Update {}".format(state))

    def set_state(self, state):  # received commands from clients
        # subclass must override and provide logic to set the device
        logger.debug("State Set {}".format(state))




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_integer import Node_Integer

import logging

logger = logging.getLogger(__name__)


class Device_Integer(Device_Base):
    def __init__(
        self, device_id=None, name=None, homie_settings=None, mqtt_settings=None
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        self.add_node(
            Node_Integer(self, id="integer", name="Integer", set_value=self.set_value)
        )

        self.start()

    def update_value(self, value):
        self.get_node("integer").update_value(value)
        logger.debug("Integer Update {}".format(value))

    def set_value(self, value):  # received commands from clients
        # subclass must override and provide logic to set the device
        logger.debug("Integer Set {}".format(value))




from .node_base import Node_Base

from homie.node.property.property_switch import Property_Switch


class Node_Switch(Node_Base):
    def __init__(
        self,
        device,
        id="switch",
        name="Switch",
        type_="switch",
        retain=True,
        qos=1,
        set_switch=None,
    ):
        super().__init__(device, id, name, type_, retain, qos)

        assert set_switch  # must provide a function to set the value of the switch

        self.add_property(Property_Switch(self, set_value=set_switch))

    def update_switch(self, onoff):
        self.get_property("switch").value = onoff




#!/usr/bin/env python

from homie.mqtt.paho_mqtt_client import PAHO_MQTT_Client
#from homie.mqtt.gmqtt_client import GMQTT_Client

MQTT_Client = PAHO_MQTT_Client


import logging

logger = logging.getLogger(__name__)

MQTT_SETTINGS = {
    "MQTT_BROKER": None,
    "MQTT_PORT": 1883,
    "MQTT_USERNAME": None,
    "MQTT_PASSWORD": None,
    "MQTT_KEEPALIVE": 60,
    "MQTT_CLIENT_ID": None,
    "MQTT_SHARE_CLIENT": None,
}

mqtt_client_count = 0


def _mqtt_validate_settings(settings):
    settings = settings.copy()

    for setting, value in MQTT_SETTINGS.items():
        if not setting in settings:
            settings[setting] = MQTT_SETTINGS[setting]
        logger.debug("MQTT Settings {} {}".format(setting, settings[setting]))

    assert settings["MQTT_BROKER"]
    assert settings["MQTT_PORT"]

    """ cannot use if two homie clients running from same pc
    if settings ['MQTT_CLIENT_ID'] is None or settings ['MQTT_SHARE_CLIENT'] is False:
        settings ['MQTT_CLIENT_ID'] = 'homiev3{:04d}'.format(mqtt_client_count) 
    """

    return settings


common_mqtt_client = None


def connect_mqtt_client(device, mqtt_settings):
    global mqtt_client_count

    mqtt_settings = _mqtt_validate_settings(mqtt_settings)

    mqtt_client = None

    if mqtt_settings["MQTT_SHARE_CLIENT"] is not True:

        logger.debug(
            "Using new MQTT client, number of instances {}".format(mqtt_client_count)
        )

        mqtt_client = MQTT_Client(mqtt_settings)
        mqtt_client.connect()
        mqtt_client_count = mqtt_client_count + 1

    else:
        logger.debug("Using common MQTT client")

        global common_mqtt_client
        if common_mqtt_client is None:
            common_mqtt_client = MQTT_Client(mqtt_settings)
            common_mqtt_client.connect()
            mqtt_client_count = mqtt_client_count + 1

        mqtt_client = common_mqtt_client

    mqtt_client.add_device(device)

    return mqtt_client

